@@ -17,6 +17,11 @@
  
     Copyright Matthias Jentsch, Vitaly Valtman, Christopher Althaus, Markus Endres 2014
 */
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
 #include <glib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -27,6 +32,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <pwd.h>
+#include <regex.h>
 
 #include "purple.h"
 #include "notify.h"
@@ -60,6 +66,8 @@
 #include "telegram-base.h"
 #include "telegram-purple.h"
 #include "msglog.h"
+#include "tgp-utils.h"
+#include "tgp-chat.h"
 
 #define _(m) m
 
@@ -67,36 +75,14 @@ PurplePlugin *_telegram_protocol = NULL;
 PurpleGroup *tggroup;
 const char *config_dir = ".telegram-purple";
 const char *pk_path = "/etc/telegram-purple/server.pub";
-void tgprpl_login_on_connected();
+
 void on_user_get_info (struct tgl_state *TLS, void *info_data, int success, struct tgl_user *U);
 
-static connection_data *get_conn_from_buddy (PurpleBuddy *buddy) {
-  connection_data *c = purple_connection_get_protocol_data (
-      purple_account_get_connection (purple_buddy_get_account (buddy)));
-  return c;
-}
-
-static const char *format_time (time_t date) {
-  struct tm *datetime = localtime(&date);
-  return purple_utf8_strftime("%Y.%m.%d %H:%M:%S", datetime);
-}
-
 static char *format_status (struct tgl_user_status *status) {
   return status->online ? "Online" : "Mobile";
 }
 
-static char *format_img_full (int imgstore) {
-  const char *br = "<br>";
-  
-  // <br>'s look ugly in Adium, but no <br> will look ugly in Pidgin
-#ifdef __ADIUM_
-  br = "";
-#endif
-  return g_strdup_printf ("%s<img id=\"%u\">",br, imgstore);
-}
-
-static char *format_service_msg (struct tgl_state *TLS, struct tgl_message *M)
-{
+static char *format_service_msg (struct tgl_state *TLS, struct tgl_message *M) {
   assert (M && M->service);
 
   char *txt_user = NULL;
@@ -158,16 +144,16 @@ static char *format_service_msg (struct tgl_state *TLS, struct tgl_message *M)
       txt_action = g_strdup_printf ("updated layer to %d", M->action.layer);
       break;
     case tgl_message_action_request_key:
-      txt_action = g_strdup_printf ("Request rekey #%016llx\n", M->action.exchange_id);
+      txt_action = g_strdup_printf ("Request rekey #%016llx", M->action.exchange_id);
       break;
     case tgl_message_action_accept_key:
-      txt_action = g_strdup_printf ("Accept rekey #%016llx\n", M->action.exchange_id);
+      txt_action = g_strdup_printf ("Accept rekey #%016llx", M->action.exchange_id);
       break;
     case tgl_message_action_commit_key:
-      txt_action = g_strdup_printf ("Commit rekey #%016llx\n", M->action.exchange_id);
+      txt_action = g_strdup_printf ("Commit rekey #%016llx", M->action.exchange_id);
       break;
     case tgl_message_action_abort_key:
-      txt_action = g_strdup_printf ("Abort rekey #%016llx\n", M->action.exchange_id);
+      txt_action = g_strdup_printf ("Abort rekey #%016llx", M->action.exchange_id);
       break;
     default:
       txt_action = NULL;
@@ -182,6 +168,28 @@ static char *format_service_msg (struct tgl_state *TLS, struct tgl_message *M)
   return txt;
 }
 
+char *format_user_status (struct tgl_user_status *status) {
+  char *when;
+  switch (status->online) {
+    case -1:
+      when = g_strdup_printf("%s", format_time (status->when));
+      break;
+    case -2:
+      when = g_strdup_printf("recently");
+      break;
+    case -3:
+      when = g_strdup_printf("last week");
+      break;
+    case -4:
+      when = g_strdup_printf("last month");
+      break;
+    default:
+      when = g_strdup ("unknown");
+      break;
+  }
+  return when;
+}
+
 static char *format_print_name (struct tgl_state *TLS, tgl_peer_id_t id, const char *a1, const char *a2, const char *a3, const char *a4) {
   const char *d[4];
   d[0] = a1; d[1] = a2; d[2] = a3; d[3] = a4;
@@ -217,7 +225,7 @@ static char *format_print_name (struct tgl_state *TLS, tgl_peer_id_t id, const c
 }
 
 static char *format_document_desc (char *type, char *caption, gint64 size) {
-  char *s = g_format_size_for_display (size);
+  char *s = tgp_g_format_size (size);
   char *msg = g_strdup_printf ("[%s] %s %s", type, caption, s);
   g_free (s);
   return msg;
@@ -226,23 +234,11 @@ static char *format_document_desc (char *type, char *caption, gint64 size) {
 static char *format_message (struct tgl_message *M) {
 
   switch (M->media.type) {
-    case tgl_message_media_audio:
-      return format_document_desc("AUDIO", "", M->media.audio.size);
-      break;
-    case tgl_message_media_audio_encr:
-      return format_document_desc("AUDIO", "", M->media.encr_audio.size);
-      break;
     case tgl_message_media_document:
       return format_document_desc("DOCUMENT", M->media.document.caption, M->media.document.size);
       break;
     case tgl_message_media_document_encr:
-      return format_document_desc("DOCUMENT", M->media.encr_document.file_name, M->media.encr_document.size);
-      break;
-    case tgl_message_media_video:
-      return format_document_desc("VIDEO", M->media.video.caption, M->media.video.size);
-      break;
-    case tgl_message_media_video_encr:
-      return format_document_desc("VIDEO", "", M->media.encr_video.size);
+      return format_document_desc("DOCUMENT", M->media.encr_document.caption, M->media.encr_document.size);
       break;
     case tgl_message_media_photo_encr:
       return format_document_desc("PHOTO", "", M->media.encr_photo.size);
@@ -259,71 +255,18 @@ static char *format_message (struct tgl_message *M) {
   }
 }
 
-
-static void tgl_do_send_unescape_message (struct tgl_state *TLS, const char *message, tgl_peer_id_t to)
-{
+static void tgl_do_send_unescape_message (struct tgl_state *TLS, const char *message, tgl_peer_id_t to) {
   gchar *raw = purple_unescape_html(message);
   tgl_do_send_message (TLS, to, raw, (int)strlen (raw), 0, 0);
   g_free(raw);
 }
 
-static tgl_peer_t *find_peer_by_name (struct tgl_state *TLS, const char *who) {
-  tgl_peer_t *peer = tgl_peer_get (TLS, TGL_MK_USER(atoi (who)));
-  if (peer) { return peer; }
-  peer = tgl_peer_get (TLS, TGL_MK_CHAT(atoi(who)));
-  if (peer) { return peer; }
-  peer = tgl_peer_get (TLS, TGL_MK_ENCR_CHAT(atoi(who)));
-  if (peer) { return peer; }
-  return NULL;
-}
-
-static int our_msg (struct tgl_state *TLS, struct tgl_message *M) {
-  //return tgl_get_peer_id(M->from_id) == TLS->our_id;
-  //return M->out;
-  return (M->flags & FLAG_SESSION_OUTBOUND) != 0;
-}
-
-static int out_msg (struct tgl_state *TLS, struct tgl_message *M) {
-  return M->out;
-}
-
-static gboolean queries_timerfunc (gpointer data) {
-  debug ("queries_timerfunc()\n");
-  connection_data *conn = data;
-  
-  if (conn->updated) {
-    conn->updated = 0;
-    write_state_file (conn->TLS);
-  }
-  return 1;
-}
-
 static void start_secret_chat (PurpleBlistNode *node, gpointer data) {
   PurpleBuddy *buddy = data;
-  connection_data *conn = purple_connection_get_protocol_data (
-                           purple_account_get_connection (purple_buddy_get_account(buddy)));
-  
-  tgl_do_create_secret_chat(conn->TLS, TGL_MK_USER(atoi (purple_buddy_get_name (buddy))),
-                            0, 0);
-}
 
-static void on_update_user_name (struct tgl_state *TLS, tgl_peer_t *user) __attribute__ ((unused));
-static void on_update_user_name (struct tgl_state *TLS, tgl_peer_t *user) {
-  p2tgl_got_alias(TLS, user->id, p2tgl_strdup_alias(user));
-}
-
-static void on_update_chat_participants (struct tgl_state *TLS, struct tgl_chat *chat) {
-  PurpleConversation *pc = purple_find_chat(tg_get_conn(TLS), tgl_get_peer_id(chat->id));
-  if (pc) {
-    purple_conv_chat_clear_users (purple_conversation_get_chat_data(pc));
-    chat_add_all_users (pc, chat);
-  }
-}
-
-static void on_update_new_user_status (struct tgl_state *TLS, void *peer) __attribute__ ((unused));
-static void on_update_new_user_status (struct tgl_state *TLS, void *peer) {
-  tgl_peer_t *p = peer;
-  p2tgl_prpl_got_user_status(TLS, p->id, &p->user.status);
+  connection_data *conn = get_conn_from_buddy (buddy);
+  const char *name = purple_buddy_get_name (buddy);
+  tgl_do_create_secret_chat (conn->TLS, TGL_MK_USER(atoi (name)), 0, 0);
 }
 
 static void update_message_received (struct tgl_state *TLS, struct tgl_message *M);
@@ -343,47 +286,44 @@ struct tgl_update_callback tgp_callback = {
 };
 
 void on_message_load_photo (struct tgl_state *TLS, void *extra, int success, char *filename) {
+  
   connection_data *conn = TLS->ev_base;
-  
-  gchar *data = NULL;
-  size_t len;
-  GError *err = NULL;
-  g_file_get_contents (filename, &data, &len, &err);
-  int imgStoreId = purple_imgstore_add_with_id (g_memdup(data, (guint)len), len, NULL);
-  used_images_add (conn, imgStoreId);
-  
-  char *image = format_img_full (imgStoreId);
-  struct tgl_message *M = extra;
-  switch (tgl_get_peer_type (M->to_id)) {
-    case TGL_PEER_CHAT:
-      debug ("PEER_CHAT\n");
-      if (!our_msg(TLS, M)) {
-        chat_add_message (TLS, M, image);
-      }
-      break;
-      
-    case TGL_PEER_USER:
-      debug ("PEER_USER\n");
-      if (out_msg(TLS, M)) {
-        p2tgl_got_im (TLS, M->to_id, image, PURPLE_MESSAGE_SEND | PURPLE_MESSAGE_IMAGES, M->date);
-      } else {
-        p2tgl_got_im (TLS, M->from_id, image, PURPLE_MESSAGE_RECV | PURPLE_MESSAGE_IMAGES, M->date);
-      }
-      break;
+
+  int imgStoreId = p2tgl_imgstore_add_with_id (filename);
+  if (imgStoreId > 0) {
+    used_images_add (conn, imgStoreId);
+    char *image = format_img_full (imgStoreId);
+    
+    struct tgl_message *M = extra;
+    switch (tgl_get_peer_type (M->to_id)) {
+      case TGL_PEER_CHAT:
+        if (!our_msg (TLS, M)) {
+          chat_add_message (TLS, M, image);
+        }
+        break;
+        
+      case TGL_PEER_USER:
+        if (out_msg (TLS, M)) {
+          p2tgl_got_im_combo (TLS, M->to_id, image, PURPLE_MESSAGE_SEND | PURPLE_MESSAGE_IMAGES, M->date);
+        } else {
+          p2tgl_got_im (TLS, M->from_id, image, PURPLE_MESSAGE_RECV | PURPLE_MESSAGE_IMAGES, M->date);
+        }
+        break;
+    }
+    g_free (image);
   }
- 
-  g_free (image);
-  conn = TLS->ev_base;
-  conn->updated = 1;
+
 }
 
 static void update_message_received (struct tgl_state *TLS, struct tgl_message *M) {
-  debug ("received message\n");
   connection_data *conn = TLS->ev_base;
-  conn->updated = 1;
-
+  
+  /* don't write to this file on every single message, which
+     can be pretty overkill on calls to tgl_do_get_difference that result in many
+     updates */
+  write_state_file_schedule (conn->TLS);
+  
   if (M->service) {
-    debug ("service message, skipping...\n");
     char *text = format_service_msg (TLS, M);
     if (text) {
       switch (tgl_get_peer_type (M->to_id)) {
@@ -397,15 +337,16 @@ static void update_message_received (struct tgl_state *TLS, struct tgl_message *
       }
       g_free (text);
     }
-    conn->updated = 1;
     return;
   }
   
-  if ((M->flags & (FLAG_MESSAGE_EMPTY | FLAG_DELETED)) || !(M->flags & FLAG_CREATED)) {
-    return;
-  }
-  if (!tgl_get_peer_type (M->to_id)) {
-    warning ("Bad msg\n");
+  if (
+      (M->flags & (FLAG_MESSAGE_EMPTY | FLAG_DELETED)) ||
+     !(M->flags & FLAG_CREATED) ||
+      !M->message ||
+      our_msg (TLS, M) ||
+      !tgl_get_peer_type (M->to_id)
+  ) {
     return;
   }
 
@@ -414,11 +355,7 @@ static void update_message_received (struct tgl_state *TLS, struct tgl_message *
     return;
   }
 
-  if (!M->message || our_msg(TLS, M)) {
-    return;
-  }
-
-  char *text = format_message(M);
+  char *text = format_message (M);
   switch (tgl_get_peer_type (M->to_id)) {
     case TGL_PEER_CHAT:
       chat_add_message (TLS, M, text);
@@ -426,7 +363,7 @@ static void update_message_received (struct tgl_state *TLS, struct tgl_message *
       
     case TGL_PEER_ENCR_CHAT:
         p2tgl_got_im (TLS, M->to_id, text, PURPLE_MESSAGE_RECV, M->date);
-        
+      
         pending_reads_add (conn->pending_reads, M->to_id);
         if (p2tgl_status_is_present (purple_account_get_active_status(conn->pa))) {
           pending_reads_send_all (conn->pending_reads, conn->TLS);
@@ -435,9 +372,8 @@ static void update_message_received (struct tgl_state *TLS, struct tgl_message *
       
     case TGL_PEER_USER:
       
-      // :TODO: figure out how to add messages from different devices to history
-      if (out_msg(TLS, M)) {
-        p2tgl_got_im (TLS, M->to_id, text, PURPLE_MESSAGE_SEND, M->date);
+      if (out_msg (TLS, M)) {
+        p2tgl_got_im_combo (TLS, M->to_id, text, PURPLE_MESSAGE_SEND, M->date);
       } else {
         p2tgl_got_im (TLS, M->from_id, text, PURPLE_MESSAGE_RECV, M->date);
         
@@ -469,10 +405,7 @@ static void update_user_handler (struct tgl_state *TLS, struct tgl_user *user, u
       p2tgl_buddy_update (TLS, (tgl_peer_t *)user, flags);
     }
     if (flags & TGL_UPDATE_PHOTO) {
-      get_user_info_data* info_data = malloc (sizeof(get_user_info_data));
-      info_data->show_info = 0;
-      info_data->peer = 0;
-      tgl_do_get_user_info (TLS, user->id, 0, on_user_get_info, info_data);
+      tgl_do_get_user_info (TLS, user->id, 0, on_user_get_info, get_user_info_data_new (0, user->id));
     }
     if (flags & TGL_UPDATE_DELETED && buddy) {
       purple_blist_remove_buddy (buddy);
@@ -480,8 +413,8 @@ static void update_user_handler (struct tgl_state *TLS, struct tgl_user *user, u
   }
 }
 
-static void write_secret_chat_cb (struct tgl_state *TLS, void *extra, int success, struct tgl_secret_chat *E) {
-  debug ("update_secret_chat_handle success=%d", success);
+static void write_secret_chat_gw (struct tgl_state *TLS, void *extra, int success, struct tgl_secret_chat *E) {
+  if (!success) { return; }
   write_secret_chat_file (TLS);
 }
 
@@ -493,7 +426,7 @@ struct accept_secret_chat_data {
 static void accept_secret_chat_cb (gpointer _data, const gchar *code) {
   struct accept_secret_chat_data *data = _data;
   
-  tgl_do_accept_encr_chat_request (data->TLS, data->U, write_secret_chat_cb, 0);
+  tgl_do_accept_encr_chat_request (data->TLS, data->U, write_secret_chat_gw, 0);
   
   g_free (data);
 }
@@ -501,7 +434,6 @@ static void accept_secret_chat_cb (gpointer _data, const gchar *code) {
 static void decline_secret_chat_cb (gpointer _data, const gchar *code) {
   struct accept_secret_chat_data *data = _data;
   
-  /* TODO: implement the api call to cancel secret chats, see tgprpl_remove_buddy */
   bl_do_encr_chat_delete (data->TLS, data->U);
   purple_blist_remove_buddy (p2tgl_buddy_find(data->TLS, data->U->id));
   
@@ -531,7 +463,7 @@ static void update_secret_chat_handler (struct tgl_state *TLS, struct tgl_secret
     
     const char* choice = purple_account_get_string (conn->pa, "accept-secret-chats", "ask");
     if (! strcmp (choice, "always")) {
-      tgl_do_accept_encr_chat_request (TLS, U, write_secret_chat_cb, 0);
+      tgl_do_accept_encr_chat_request (TLS, U, write_secret_chat_gw, 0);
       
     } else if (! strcmp(choice, "ask")) {
       PurpleBuddy *who = p2tgl_buddy_find (TLS, TGL_MK_USER(U->user_id));
@@ -564,24 +496,19 @@ static void update_secret_chat_handler (struct tgl_state *TLS, struct tgl_secret
 
 static void update_chat_handler (struct tgl_state *TLS, struct tgl_chat *chat, unsigned flags) {
   PurpleChat *ch = p2tgl_chat_find (TLS, chat->id);
-  
+
   if (flags & TGL_UPDATE_CREATED) {
-    if (!ch) {
-      ch = p2tgl_chat_new (TLS, chat);
-      purple_blist_add_chat(ch, NULL, NULL);
-    }
+    tgl_do_get_chat_info (TLS, chat->id, 0, on_chat_get_info, 0);
   }
   if (flags & TGL_UPDATE_TITLE && ch) {
     purple_blist_alias_chat (ch, chat->print_title);
   }
   if (flags & (TGL_UPDATE_MEMBERS | TGL_UPDATE_ADMIN) && ch) {
-    on_update_chat_participants (TLS, chat);
+    chat_users_update (TLS, chat);
   }
   if (flags & TGL_UPDATE_DELETED && ch) {
     purple_blist_remove_chat (ch);
   }
-  
-  // TODO: check if user is a member of the current chat and don't display the chat in that case
 }
 
 static void update_user_typing (struct tgl_state *TLS, struct tgl_user *U, enum tgl_typing_status status) {
@@ -590,15 +517,6 @@ static void update_user_typing (struct tgl_state *TLS, struct tgl_user *U, enum
   }
 }
 
-PurpleNotifyUserInfo *create_user_notify_info(struct tgl_user *usr) {
-  PurpleNotifyUserInfo *info = purple_notify_user_info_new();
-  purple_notify_user_info_add_pair(info, "First name", usr->first_name);
-  purple_notify_user_info_add_pair(info, "Last name", usr->last_name);
-  purple_notify_user_info_add_pair(info, "Phone", usr->phone);
-  purple_notify_user_info_add_pair(info, "Status", usr->status.online == 1 ? "Online" : "Offline");
-  return info;
-}
-
 static void on_contact_added (struct tgl_state *TLS,void *callback_extra, int success, int size, struct tgl_user *users[]) {
   PurpleBuddy *buddy = callback_extra;
   
@@ -609,91 +527,52 @@ static void on_contact_added (struct tgl_state *TLS,void *callback_extra, int su
 }
 
 static void on_userpic_loaded (struct tgl_state *TLS, void *extra, int success, char *filename) {
-  if (!success) {
-    struct download_desc *dld = extra;
-    struct tgl_user *U = dld->data;
-    warning ("Can not load userpic for user %s %s\n", U->first_name, U->last_name);
-  }
   connection_data *conn = TLS->ev_base;
-
-  gchar *data = NULL;
-  size_t len;
-  GError *err = NULL;
-  g_file_get_contents (filename, &data, &len, &err);
-  
-  int imgStoreId = purple_imgstore_add_with_id (g_memdup(data, (guint)len), len, NULL);
-  used_images_add (conn, imgStoreId);
   
   struct download_desc *dld = extra;
   struct tgl_user *U = dld->data;
+  tgl_peer_t *P = tgl_peer_get (TLS, dld->get_user_info_data->peer);
   
-  if (imgStoreId <= 0) {
-    warning ("Can not load userpic for user %s %s\n", U->first_name, U->last_name);
+  if (!success || !P) {
+    warning ("Can not load userpic for user %s %s", U->first_name, U->last_name);
+    goto fin;
   }
+  
+  int imgStoreId = p2tgl_imgstore_add_with_id (filename);
+  if (imgStoreId > 0) {
+    used_images_add (conn, imgStoreId);
 
-  char *who = g_strdup_printf ("%d", tgl_get_peer_id (U->id));
-  if (dld->get_user_info_data->show_info == 1) {
-    PurpleNotifyUserInfo *info = create_user_notify_info(U);
+    p2tgl_buddy_icons_set_for_user (conn->pa, &P->id, filename);
     
-    if (dld->get_user_info_data->peer && dld->get_user_info_data->peer->encr_chat.first_key_sha[0]) {
-      
-      // display secret key
-      int sha1key_store_id = generate_ident_icon (conn->TLS, dld->get_user_info_data->peer->encr_chat.first_key_sha);
-      if (sha1key_store_id != -1) {
-        char *ident_icon = format_img_full (sha1key_store_id);
-        purple_notify_user_info_add_pair (info, "Secret key", ident_icon);
-        g_free(ident_icon);
-      }
-      
-      g_free (who);
-      who = g_strdup_printf ("%d", tgl_get_peer_id (dld->get_user_info_data->peer->id));
+    if (dld->get_user_info_data->show_info == 1) {
+      PurpleNotifyUserInfo *info = p2tgl_notify_peer_info_new (TLS, P);
+      p2tgl_notify_userinfo (TLS, P->id, info, NULL, NULL);
     }
-    
-    char *profile_image = profile_image = format_img_full (imgStoreId);
-    purple_notify_user_info_add_pair (info, "Profile image", profile_image);
-    purple_notify_userinfo (conn->gc, who, info, NULL, NULL);
-    
-    g_free (profile_image);
   }
-  if (dld->get_user_info_data->peer) {
-     char *id = g_strdup_printf ("%d", tgl_get_peer_id (dld->get_user_info_data->peer->id));
-     gchar *img_data = NULL;
-     size_t len;
-     GError *err = NULL;
-     g_file_get_contents (filename, &img_data, &len, &err);
-     purple_buddy_icons_set_for_user(conn->pa, id, img_data, len, NULL);
-     g_free (id);
-  }
-  purple_buddy_icons_set_for_user(conn->pa, who, data, len, NULL);
-  g_free(who);
-  g_free(dld->get_user_info_data);
+  
+fin:
+  free (dld->get_user_info_data);
+  free (dld);
 }
 
 void on_user_get_info (struct tgl_state *TLS, void *info_data, int success, struct tgl_user *U)
 {
+  get_user_info_data *user_info_data = (get_user_info_data *)info_data;
+  tgl_peer_t *P = tgl_peer_get (TLS, user_info_data->peer);
+  
   if (! success) {
-    warning ("on_user_get_info not successfull, aborting...\n");
+    warning ("on_user_get_info not successfull, aborting...");
     return;
   }
   
-  get_user_info_data *user_info_data = (get_user_info_data *)info_data;
   if (U->photo.sizes_num == 0) {
+    // No profile pic to load, display it right away
     if (user_info_data->show_info) {
-      PurpleNotifyUserInfo *info = create_user_notify_info(U);
-      if (user_info_data->peer && user_info_data->peer->encr_chat.first_key_sha[0]) {
-        // display secret key
-        int sha1key_store_id = generate_ident_icon (TLS, user_info_data->peer->encr_chat.first_key_sha);
-        if (sha1key_store_id != -1) {
-          char *ident_icon = format_img_full (sha1key_store_id);
-          purple_notify_user_info_add_pair (info, "Secret key", ident_icon);
-          g_free (ident_icon);
-        }
-        p2tgl_notify_userinfo (TLS, user_info_data->peer->id, info, NULL, NULL);
-      } else {
-	      p2tgl_notify_userinfo (TLS, U->id, info, NULL, NULL);
-      }
+      PurpleNotifyUserInfo *info = p2tgl_notify_peer_info_new (TLS, P);
+      p2tgl_notify_userinfo (TLS, P->id, info, NULL, NULL);
     }
-    g_free(user_info_data);
+    g_free (user_info_data);
+  
   } else {
     struct download_desc *dld = malloc (sizeof(struct download_desc));
     dld->data = U;
@@ -703,55 +582,36 @@ void on_user_get_info (struct tgl_state *TLS, void *info_data, int success, stru
 }
 
 void on_chat_get_info (struct tgl_state *TLS, void *extra, int success, struct tgl_chat *C) {
-  assert (success);
-  
-  debug ("on_chat_joined(%d)\n", tgl_get_peer_id (C->id));
-  connection_data *conn = TLS->ev_base;
-
-  PurpleConversation *conv;
-  if (!(conv = purple_find_chat (conn->gc, tgl_get_peer_id(C->id)))) {
-    // chat conversation is not existing, create it
-    conv = serv_got_joined_chat (conn->gc, tgl_get_peer_id(C->id), C->title);
-  }
-  purple_conv_chat_clear_users (purple_conversation_get_chat_data(conv));
-  chat_add_all_users (conv, C);
-  
-  struct message_text *mt = 0;
-  while ((mt = g_queue_pop_head (conn->new_messages))) {
-    if (!chat_add_message(TLS, mt->M, mt->text)) {
-      warning ("WARNING, chat %d still not existing... \n", tgl_get_peer_id (C->id));
-      break;
-    }
-    if (mt->text) {
-      g_free (mt->text);
-    }
-    free (mt);
+  if (!success || !chat_is_member (TLS->our_id, C)) {
+    return;
   }
   
-  gchar *name = g_strdup_printf ("%d", tgl_get_peer_id (C->id));
-  g_hash_table_remove (conn->joining_chats, name);
-  g_free (name);
+  PurpleChat *PC = p2tgl_chat_find (TLS, C->id);
+  if (!PC) {
+    PC = p2tgl_chat_new (TLS, C);
+    purple_blist_add_chat (PC, NULL, NULL);
+  }
+  
+  chat_users_update (TLS, C);
 }
 
 void on_ready (struct tgl_state *TLS) {
-  debug ("on_ready().\n");
+  debug ("on_ready().");
   connection_data *conn = TLS->ev_base;
   
-  purple_connection_set_state(conn->gc, PURPLE_CONNECTED);
-  purple_connection_set_display_name(conn->gc, purple_account_get_username(conn->pa));
-  purple_blist_add_account(conn->pa);
+  purple_connection_set_state (conn->gc, PURPLE_CONNECTED);
+  purple_connection_set_display_name (conn->gc, purple_account_get_username(conn->pa));
+  purple_blist_add_account (conn->pa);
   tggroup = purple_find_group ("Telegram");
   if (tggroup == NULL) {
     tggroup = purple_group_new ("Telegram");
     purple_blist_add_group (tggroup, NULL);
   }
   
-  debug ("seq = %d, pts = %d\n", TLS->seq, TLS->pts);
+  debug ("seq = %d, pts = %d", TLS->seq, TLS->pts);
   tgl_do_get_difference (TLS, 0, 0, 0);
   tgl_do_get_dialog_list (TLS, 0, 0);
   tgl_do_update_contact_list (TLS, 0, 0);
-  
-  conn->timer = purple_timeout_add (5000, queries_timerfunc, conn);
 }
 
 static const char *tgprpl_list_icon (PurpleAccount * acct, PurpleBuddy * buddy) {
@@ -759,24 +619,27 @@ static const char *tgprpl_list_icon (PurpleAccount * acct, PurpleBuddy * buddy)
 }
 
 static void tgprpl_tooltip_text (PurpleBuddy * buddy, PurpleNotifyUserInfo * info, gboolean full) {
-  debug ("tgprpl_tooltip_text()\n", buddy->name);
+  debug ("tgprpl_tooltip_text()", buddy->name);
   
   tgl_peer_id_t *peer = purple_buddy_get_protocol_data(buddy);
   if (!peer) {
-    purple_notify_user_info_add_pair_plaintext(info, "Status", "Offline");
+    purple_notify_user_info_add_pair (info, "Status", "Offline");
     return;
   }
   tgl_peer_t *P = tgl_peer_get (get_conn_from_buddy (buddy)->TLS, *peer);
   if (!P) {
-    warning ("tgprpl_tooltip_text: warning peer with id %d not found in tree.\n", peer->id);
+    warning ("tgprpl_tooltip_text: warning peer with id %d not found in tree.", peer->id);
     return;
   }
-  purple_notify_user_info_add_pair_plaintext (info, "Status", format_status(&P->user.status));
-  purple_notify_user_info_add_pair_plaintext (info, "Last seen: ", format_time(P->user.status.when));
+  purple_notify_user_info_add_pair (info, "Status", format_status(&P->user.status));
+  
+  gchar *status = format_user_status (&P->user.status);
+  purple_notify_user_info_add_pair (info, "last online: ", status);
+  g_free (status);
 }
 
 static GList *tgprpl_status_types (PurpleAccount * acct) {
-  debug ("tgprpl_status_types()\n");
+  debug ("tgprpl_status_types()");
   GList *types = NULL;
   PurpleStatusType *type;
   
@@ -803,7 +666,7 @@ static GList *tgprpl_status_types (PurpleAccount * acct) {
 }
 
 static GList* tgprpl_blist_node_menu (PurpleBlistNode *node) {
-  purple_debug_info (PLUGIN_ID, "tgprpl_blist_node_menu()\n");
+  purple_debug_info (PLUGIN_ID, "tgprpl_blist_node_menu()");
   
   GList* menu = NULL;
   if (PURPLE_BLIST_NODE_IS_BUDDY(node)) {
@@ -817,7 +680,7 @@ static GList* tgprpl_blist_node_menu (PurpleBlistNode *node) {
 }
 
 static GList *tgprpl_chat_join_info (PurpleConnection * gc) {
-  debug ("tgprpl_chat_join_info()\n");
+  debug ("tgprpl_chat_join_info()");
   struct proto_chat_entry *pce;
   
   pce = g_new0(struct proto_chat_entry, 1);
@@ -828,25 +691,25 @@ static GList *tgprpl_chat_join_info (PurpleConnection * gc) {
 }
 
 static void tgprpl_login (PurpleAccount * acct) {
-  debug ("tgprpl_login()\n");
-  PurpleConnection *gc = purple_account_get_connection(acct);
-  char const *username = purple_account_get_username(acct);
+  debug ("tgprpl_login()");
+  PurpleConnection *gc = purple_account_get_connection (acct);
+  char const *username = purple_account_get_username (acct);
   
   struct tgl_state *TLS = tgl_state_alloc ();
   
   const char *dir = config_dir;
-  struct passwd *pw = getpwuid(getuid());
+  struct passwd *pw = getpwuid (getuid ());
   size_t len = strlen (dir) + strlen (pw->pw_dir) + 2 + strlen (username);
   TLS->base_path = malloc (len);
   snprintf (TLS->base_path, len, "%s/%s/%s", pw->pw_dir, dir, username);
   debug ("base configuration path: '%s'", TLS->base_path);
-  g_mkdir_with_parents(TLS->base_path, 0700);
+  g_mkdir_with_parents (TLS->base_path, 0700);
   
   len += strlen ("/downloads");
   char *ddir = malloc (len);
   sprintf (ddir, "%s/downloads", TLS->base_path);
   tgl_set_download_directory (TLS, ddir);
-  g_mkdir_with_parents(ddir, 0700);
+  g_mkdir_with_parents (ddir, 0700);
   free (ddir);
   
   tgl_set_verbosity (TLS, 4);
@@ -870,15 +733,14 @@ static void tgprpl_login (PurpleAccount * acct) {
 }
 
 static void tgprpl_close (PurpleConnection * gc) {
-  debug ("tgprpl_close()\n");
+  debug ("tgprpl_close()");
   connection_data *conn = purple_connection_get_protocol_data (gc);
-  purple_timeout_remove (conn->timer);
   
   connection_data_free (conn);
 }
 
 static int tgprpl_send_im (PurpleConnection * gc, const char *who, const char *message, PurpleMessageFlags flags) {
-  debug ("tgprpl_send_im()\n");
+  debug ("tgprpl_send_im()");
   connection_data *conn = purple_connection_get_protocol_data(gc);
  
   // this is part of a workaround to support clients without
@@ -911,7 +773,7 @@ static int tgprpl_send_im (PurpleConnection * gc, const char *who, const char *m
 }
 
 static unsigned int tgprpl_send_typing (PurpleConnection * gc, const char *who, PurpleTypingState typing) {
-  debug ("tgprpl_send_typing()\n");
+  debug ("tgprpl_send_typing()");
   int id = atoi (who);
   connection_data *conn = purple_connection_get_protocol_data(gc);
   tgl_peer_t *U = tgl_peer_get (conn->TLS, TGL_MK_USER (id));
@@ -926,48 +788,33 @@ static unsigned int tgprpl_send_typing (PurpleConnection * gc, const char *who,
 }
 
 static void tgprpl_get_info (PurpleConnection * gc, const char *who) {
-  debug ("tgprpl_get_info()\n");
+  debug ("tgprpl_get_info()");
   connection_data *conn = purple_connection_get_protocol_data(gc);
   
   tgl_peer_t *peer = find_peer_by_name (conn->TLS, who);
   if (! peer) { return; }
   
-  get_user_info_data* info_data = malloc (sizeof(get_user_info_data));
-  info_data->show_info = 1;
-  info_data->peer = peer;
+  get_user_info_data* info_data = get_user_info_data_new (1, peer->id);
   
   switch (tgl_get_peer_type (peer->id)) {
     case TGL_PEER_USER:
     case TGL_PEER_CHAT:
       tgl_do_get_user_info (conn->TLS, peer->id, 0, on_user_get_info, info_data);
       break;
+      
     case TGL_PEER_ENCR_CHAT: {
-      tgl_peer_t *parent_peer;
-      
-      if (peer->encr_chat.state == sc_waiting) {
-        PurpleNotifyUserInfo *info = purple_notify_user_info_new();
-        purple_notify_user_info_add_pair (info, "", "Waiting for user to get online ...");
-        p2tgl_notify_userinfo (conn->TLS, peer->id, info, NULL, NULL);
-        return;
-      }
-      
-      if (peer->encr_chat.admin_id == conn->TLS->our_id){
-        parent_peer = tgl_peer_get (conn->TLS, TGL_MK_USER(peer->encr_chat.user_id));
-      } else {
-        parent_peer = tgl_peer_get (conn->TLS, TGL_MK_USER(peer->encr_chat.admin_id));
-      }
-      if (parent_peer){
-        tgl_do_get_user_info(conn->TLS, parent_peer->id, 0, on_user_get_info, info_data);
+      tgl_peer_t *parent_peer = tgp_encr_chat_get_partner(conn->TLS, &peer->encr_chat);
+      if (parent_peer) {
+        tgl_do_get_user_info (conn->TLS, parent_peer->id, 0, on_user_get_info, info_data);
       }
       break;
     }
-      
   }
 }
 
 static void tgprpl_set_status (PurpleAccount * acct, PurpleStatus * status) {
-  debug ("tgprpl_set_status(%s)\n", purple_status_get_name (status));
-  debug ("tgprpl_set_status(currstatus=%s)\n", purple_status_get_name(purple_account_get_active_status(acct)));
+  debug ("tgprpl_set_status(%s)", purple_status_get_name (status));
+  debug ("tgprpl_set_status(currstatus=%s)", purple_status_get_name(purple_account_get_active_status(acct)));
 
   PurpleConnection *gc = purple_account_get_connection(acct);
   if (!gc) { return; }
@@ -986,7 +833,7 @@ static void tgprpl_add_buddy (PurpleConnection * gc, PurpleBuddy * buddy, Purple
 }
 
 static void tgprpl_remove_buddy (PurpleConnection * gc, PurpleBuddy * buddy, PurpleGroup * group) {
-  debug ("tgprpl_remove_buddy()\n");
+  debug ("tgprpl_remove_buddy()");
   if (!buddy) { return; }
 
   connection_data *conn = purple_connection_get_protocol_data (gc);
@@ -996,7 +843,6 @@ static void tgprpl_remove_buddy (PurpleConnection * gc, PurpleBuddy * buddy, Pur
     // telegram peer not existing, orphaned buddy
     return;
   }
-  
   switch (tgl_get_peer_type(peer->id)) {
     case TGL_PEER_ENCR_CHAT:
       /* TODO: implement the api call cancel secret chats. Currently the chat will only be marked as 
@@ -1011,53 +857,45 @@ static void tgprpl_remove_buddy (PurpleConnection * gc, PurpleBuddy * buddy, Pur
   
 }
 
-static void tgprpl_add_deny (PurpleConnection * gc, const char *name){
-  debug ("tgprpl_add_deny()\n");
+static void tgprpl_add_deny (PurpleConnection * gc, const char *name) {
+  debug ("tgprpl_add_deny()");
 }
 
-static void tgprpl_rem_deny (PurpleConnection * gc, const char *name){
-  debug ("tgprpl_rem_deny()\n");
+static void tgprpl_rem_deny (PurpleConnection * gc, const char *name) {
+  debug ("tgprpl_rem_deny()");
 }
 
 static void tgprpl_chat_join (PurpleConnection * gc, GHashTable * data) {
-  debug ("tgprpl_chat_join()\n");
+  debug ("tgprpl_chat_join()");
   
   connection_data *conn = purple_connection_get_protocol_data (gc);
-  const char *groupname = g_hash_table_lookup (data, "subject");
-  
-  char *id = g_hash_table_lookup(data, "id");
-  if (!id) {
-    warning ("Got no chat id, aborting...\n");
-    return;
-  }
-  if (!purple_find_chat(gc, atoi(id))) {
-    tgl_do_get_chat_info (conn->TLS, TGL_MK_CHAT(atoi(id)), 0, on_chat_get_info, 0);
-  } else {
-    serv_got_joined_chat(conn->gc, atoi(id), groupname);
+  int id = atoi (g_hash_table_lookup (data, "id"));
+  if (id) {
+    chat_show (conn->gc, id);
   }
 }
 
 static char *tgprpl_get_chat_name (GHashTable * data) {
-  debug ("tgprpl_get_chat_name()\n");
+  debug ("tgprpl_get_chat_name()");
   return g_strdup(g_hash_table_lookup(data, "subject"));
 }
 
 static PurpleXfer *tgprpl_new_xfer (PurpleConnection * gc, const char *who) {
-  debug ("tgprpl_new_xfer()\n");
+  debug ("tgprpl_new_xfer()");
   return (PurpleXfer *)NULL;
 }
 
 static void tgprpl_send_file (PurpleConnection * gc, const char *who, const char *file) {
-  debug ("tgprpl_send_file()\n");
+  debug ("tgprpl_send_file()");
 }
 
 static GHashTable *tgprpl_chat_info_deflt (PurpleConnection * gc, const char *chat_name) {
-  debug ("tgprpl_chat_info_defaults()\n");
+  debug ("tgprpl_chat_info_defaults()");
   return NULL;
 }
 
 static void tgprpl_chat_invite (PurpleConnection * gc, int id, const char *message, const char *name) {
-  debug ("tgprpl_chat_invite()\n");
+  debug ("tgprpl_chat_invite()");
 
   connection_data *conn = purple_connection_get_protocol_data (gc);
   tgl_peer_t *chat = tgl_peer_get(conn->TLS, TGL_MK_CHAT (id));
@@ -1072,35 +910,29 @@ static void tgprpl_chat_invite (PurpleConnection * gc, int id, const char *messa
 }
 
 static int tgprpl_send_chat (PurpleConnection * gc, int id, const char *message, PurpleMessageFlags flags) {
-  debug ("tgprpl_send_chat()\n");
+  debug ("tgprpl_send_chat()");
   connection_data *conn = purple_connection_get_protocol_data (gc);
   tgl_do_send_unescape_message (conn->TLS, message, TGL_MK_CHAT(id));
-  
-  /* Pidgin won't display the written message if we don't call this, Adium will display it twice 
-     if we call it, so we don't do it for the adium Plugin.
-     TODO: there has to be a better way to do this, figure out how.
-   */
-#ifndef __ADIUM_
+
   p2tgl_got_chat_in(conn->TLS, TGL_MK_CHAT(id), TGL_MK_USER(conn->TLS->our_id), message,
     PURPLE_MESSAGE_RECV, time(NULL));
-#endif
   return 1;
 }
 
 static void tgprpl_group_buddy (PurpleConnection * gc, const char *who, const char *old_group, const char *new_group) {
-  debug ("tgprpl_group_buddy()\n");
+  debug ("tgprpl_group_buddy()");
 }
 
 static void tgprpl_rename_group (PurpleConnection * gc, const char *old_name, PurpleGroup * group, GList * moved_buddies) {
-  debug ("tgprpl_rename_group()\n");
+  debug ("tgprpl_rename_group()");
 }
 
-static void tgprpl_convo_closed (PurpleConnection * gc, const char *who){
-  debug ("tgprpl_convo_closed()\n");
+static void tgprpl_convo_closed (PurpleConnection * gc, const char *who) {
+  debug ("tgprpl_convo_closed()");
 }
 
 static void tgprpl_set_buddy_icon (PurpleConnection * gc, PurpleStoredImage * img) {
-  debug ("tgprpl_set_buddy_icon()\n");
+  debug ("tgprpl_set_buddy_icon()");
   
   connection_data *conn = purple_connection_get_protocol_data (gc);
   if (purple_imgstore_get_filename (img)) {
@@ -1114,12 +946,12 @@ static void tgprpl_set_buddy_icon (PurpleConnection * gc, PurpleStoredImage * im
 }
 
 static gboolean tgprpl_can_receive_file (PurpleConnection * gc, const char *who) {
-  debug ("tgprpl_can_receive_file()\n");
+  debug ("tgprpl_can_receive_file()");
   return 0;
 }
 
 static gboolean tgprpl_offline_message (const PurpleBuddy * buddy) {
-  debug ("tgprpl_offline_message()\n");
+  debug ("tgprpl_offline_message()");
   return 0;
 }
 
@@ -1178,10 +1010,10 @@ static PurplePluginProtocolInfo prpl_info = {
   tgprpl_rename_group,
   NULL,                    // buddy_free
   tgprpl_convo_closed,     
-  purple_normalize_nocase, 
+  NULL,                    // normalize
   tgprpl_set_buddy_icon,
   NULL,                    // remove_group
-  NULL,                    // get_cb_real_name
+  NULL,
   NULL,                    // set_chat_topic
   NULL,                    // find_blist_chat
   NULL,                    // roomlist_get_list
@@ -1250,7 +1082,7 @@ static PurplePluginInfo plugin_info = {
   PURPLE_PRIORITY_DEFAULT,
   PLUGIN_ID,
   "Telegram",
-  TG_VERSION " libtgl: " TGL_VERSION,
+  PACKAGE_VERSION " libtgl: " TGL_VERSION,
   "Telegram",
   TG_DESCRIPTION,
   TG_AUTHOR,
