@@ -25,7 +25,6 @@
 #define PLUGIN_ID "prpl-telegram"
 #define TG_AUTHOR "Matthias Jentsch <mtthsjntsch@gmail.com>, Vitaly Valtman, Christopher Althaus <althaus.christopher@gmail.com>, Markus Endres <endresma45241@th-nuernberg.de>. Based on libtgl by Vitaly Valtman."
 #define TG_DESCRIPTION "Telegram protocol."
-#define TG_VERSION "0.6"
 #define TG_BUILD "11"
 
 #include <notify.h>
@@ -42,5 +41,6 @@ void on_ready (struct tgl_state *TLS);
 extern const char *pk_path;
 extern const char *config_dir;
 extern PurplePlugin *_telegram_protocol;
+char *format_user_status (struct tgl_user_status *status);
 
 #endif
