@@ -20,6 +20,9 @@
 #include "telegram-purple.h"
 #include "tgp-2prpl.h"
 #include "tgp-structs.h"
+#include "telegram-purple.h"
+#include "tgp-utils.h"
+#include "telegram-base.h"
 
 #include <server.h>
 #include <tgl.h>
@@ -48,7 +51,7 @@ int p2tgl_status_is_present (PurpleStatus *status)
   return !(strcmp (name, "unavailable") == 0 || strcmp (name, "away") == 0);
 }
 
-static PurpleChat *blist_find_chat_by_hasht_cond(PurpleConnection *gc,
+static PurpleChat *blist_find_chat_by_hasht_cond (PurpleConnection *gc,
     int (*fn)(GHashTable *hasht, void *data), void *data) {
   PurpleAccount *account = purple_connection_get_account(gc);
   PurpleBlistNode *node = purple_blist_get_root();
@@ -91,9 +94,9 @@ void p2tgl_got_chat_left (struct tgl_state *TLS, tgl_peer_id_t chat) {
 }
 
 void p2tgl_got_chat_in (struct tgl_state *TLS, tgl_peer_id_t chat, tgl_peer_id_t who, const char *message, int flags, time_t when) {
-  char *name = p2tgl_peer_strdup_id(who);
+  char *name = p2tgl_peer_strdup_id (who);
   
-  serv_got_chat_in(tg_get_conn(TLS), tgl_get_peer_id (chat), name, flags, message, when);
+  serv_got_chat_in (tg_get_conn(TLS), tgl_get_peer_id (chat), name, flags, message, when);
   
   g_free (name);
 }
@@ -114,6 +117,31 @@ void p2tgl_got_im (struct tgl_state *TLS, tgl_peer_id_t who, const char *msg, in
   g_free (name);
 }
 
+void p2tgl_got_im_combo (struct tgl_state *TLS, tgl_peer_id_t who, const char *msg, int flags, time_t when) {
+  
+  /* 
+     Outgoing messages are not well supported in different libpurple clients, 
+     p2tgl_conversation_write should have the best among different versions. Unfortunately 
+     this causes buggy formatting in Adium, so we don't use this workaround in that case.
+   
+     NOTE: Outgoing messages will not work in Adium <= 1.6.0, there is no way to print outgoing
+     messages in those versions at all.
+   */
+#ifndef __ADIUM_
+  if (flags & PURPLE_MESSAGE_SEND) {
+    PurpleConversation *conv = p2tgl_find_conversation_with_account (TLS, who);
+    if (!conv) {
+      conv = p2tgl_conversation_new(TLS, who);
+    }
+    p2tgl_conversation_write (conv, who, msg, PURPLE_MESSAGE_SEND, when);
+    return;
+  }
+#endif
+
+  p2tgl_got_im (TLS, who, msg, flags, when);
+}
+
+
 void p2tgl_got_typing (struct tgl_state *TLS, tgl_peer_id_t user, int timeout) {
   char *who = g_strdup_printf("%d", tgl_get_peer_id(user));
   
@@ -132,6 +160,39 @@ PurpleBuddy *p2tgl_buddy_find (struct tgl_state *TLS, tgl_peer_id_t user) {
   return b;
 }
 
+PurpleConversation *p2tgl_find_conversation_with_account (struct tgl_state *TLS, tgl_peer_id_t peer) {
+  int type = PURPLE_CONV_TYPE_IM;
+  if (tgl_get_peer_type (peer) == TGL_PEER_CHAT) {
+    type = PURPLE_CONV_TYPE_CHAT;
+  }
+  char *who = g_strdup_printf("%d", tgl_get_peer_id(peer));
+  
+  PurpleConversation *conv = purple_find_conversation_with_account (type, who, tg_get_acc(TLS));
+  
+  g_free (who);
+  return conv;
+}
+
+void p2tgl_conversation_write (PurpleConversation *conv, tgl_peer_id_t who, const char *message, int flags, int date) {
+  char *name = p2tgl_peer_strdup_id (who);
+  
+  purple_conv_im_write(purple_conversation_get_im_data(conv), name, message, flags, date);
+  // purple_conversation_write (conv, name, message, flags, date);
+  
+  g_free (name);
+}
+
+PurpleConversation *p2tgl_conversation_new (struct tgl_state *TLS, tgl_peer_id_t who) {
+  int type =  tgl_get_peer_type (who) == TGL_PEER_CHAT ? PURPLE_CONV_TYPE_CHAT : PURPLE_CONV_TYPE_IM;
+  
+  char *name = p2tgl_peer_strdup_id (who);
+  PurpleConversation *conv = purple_conversation_new(type, tg_get_acc(TLS), name);
+  g_free (name);
+  
+  return conv;
+}
+
+
 PurpleBuddy *p2tgl_buddy_new  (struct tgl_state *TLS, tgl_peer_t *user) {
   char *alias = p2tgl_strdup_alias (user);
   char *name  = p2tgl_peer_strdup_id (user->id);
@@ -149,7 +210,7 @@ PurpleBuddy *p2tgl_buddy_update (struct tgl_state *TLS, tgl_peer_t *user, unsign
     b = p2tgl_buddy_new (TLS, user);
   }
   if (flags & (TGL_UPDATE_NAME | TGL_UPDATE_REAL_NAME | TGL_UPDATE_USERNAME)) {
-    debug ("Update username for id%d (name %s %s)\n", tgl_get_peer_id (user->id), user->user.first_name, user->user.last_name);
+    debug ("Update username for id%d (name %s %s)", tgl_get_peer_id (user->id), user->user.first_name, user->user.last_name);
     char *alias = p2tgl_strdup_alias (user);
     purple_blist_alias_buddy(b, alias);
     g_free (alias);
@@ -174,36 +235,10 @@ void p2tgl_prpl_got_set_status_offline (struct tgl_state *TLS, tgl_peer_id_t use
 }
 
 void p2tgl_prpl_got_user_status (struct tgl_state *TLS, tgl_peer_id_t user, struct tgl_user_status *status) {
-  
   if (status->online == 1) {
-    char *name = p2tgl_peer_strdup_id (user);
-    purple_prpl_got_user_status (tg_get_acc(TLS), name, "available", NULL);
-    g_free (name);
+    p2tgl_prpl_got_set_status_offline(TLS, user);
   } else {
-    char *name = p2tgl_peer_strdup_id (user);
-    char *when;
-    switch (status->online) {
-    case -1:
-      when = g_strdup_printf("%d", status->when);
-      break;
-    case -2:
-      when = g_strdup_printf("recently");
-      break;
-    case -3:
-      when = g_strdup_printf("last week");
-      break;
-    case -4:
-      when = g_strdup_printf("last month");
-      break;
-    default:
-      when = g_strdup ("unknown");
-      break;
-    }
-  
-    purple_prpl_got_user_status (tg_get_acc(TLS), name, "mobile", "last online", when, NULL);
-  
-    g_free(name);
-    g_free(when);
+    p2tgl_prpl_got_set_status_mobile(TLS, user);
   }
 }
 
@@ -259,6 +294,7 @@ void *p2tgl_notify_userinfo(struct tgl_state *TLS, tgl_peer_id_t user, PurpleNot
   handle = purple_notify_userinfo(tg_get_conn(TLS), name, user_info, cb, user_data);
   
   g_free(name);
+  g_free(user_info);
   return handle;
 }
 
@@ -269,3 +305,103 @@ void p2tgl_blist_alias_buddy (PurpleBuddy *buddy, struct tgl_user *user) {
   
   g_free(name);
 }
+
+PurpleNotifyUserInfo *p2tgl_notify_user_info_new (struct tgl_user *U) {
+  PurpleNotifyUserInfo *info = purple_notify_user_info_new();
+  
+  if (str_not_empty(U->first_name) && str_not_empty(U->last_name)) {
+    purple_notify_user_info_add_pair (info, "First name", U->first_name);
+    purple_notify_user_info_add_pair (info, "Last name", U->last_name);
+  } else {
+    purple_notify_user_info_add_pair (info, "Name", U->print_name);
+  }
+  
+  if (str_not_empty (U->username)) {
+    purple_notify_user_info_add_pair (info, "Username", U->username);
+  }
+  
+  char *status = format_user_status (&U->status);
+  purple_notify_user_info_add_pair (info, "Last seen", status);
+  g_free (status);
+
+  if (str_not_empty (U->phone)) {
+    char *phone = g_strdup_printf("+%s", U->phone);
+    purple_notify_user_info_add_pair (info, "Phone", phone);
+    g_free (phone);
+  }
+ 
+  return info;
+}
+
+PurpleNotifyUserInfo *p2tgl_notify_encrypted_chat_info_new (struct tgl_state *TLS,
+                                                            struct tgl_secret_chat *secret, struct tgl_user *U) {
+  
+  PurpleNotifyUserInfo *info = p2tgl_notify_user_info_new (U);
+  
+  if (secret->state == sc_waiting) {
+    purple_notify_user_info_add_pair (info, "", "Waiting for user to get online ...");
+    return info;
+  }
+  
+  const char *ttl_key = "Self destructiom timer";
+  if (secret->ttl) {
+    char *ttl = g_strdup_printf ("%d", secret->ttl);
+    purple_notify_user_info_add_pair (info, ttl_key, ttl);
+    g_free (ttl);
+  } else {
+    purple_notify_user_info_add_pair (info, ttl_key, "Off");
+  }
+  
+  if (secret->first_key_sha[0]) {
+    int sha1key_store_id = generate_ident_icon (TLS, secret->first_key_sha);
+    if (sha1key_store_id != -1) {
+      char *ident_icon = format_img_full (sha1key_store_id);
+      purple_notify_user_info_add_pair (info, "Secret key", ident_icon);
+      g_free(ident_icon);
+    }
+  }
+  
+  return info;
+}
+
+PurpleNotifyUserInfo *p2tgl_notify_peer_info_new (struct tgl_state *TLS, tgl_peer_t *P) {
+  switch (tgl_get_peer_type (P->id)) {
+    case TGL_PEER_ENCR_CHAT: {
+      struct tgl_secret_chat *chat = &P->encr_chat;
+      tgl_peer_t *partner = tgp_encr_chat_get_partner (TLS, chat);
+      return p2tgl_notify_encrypted_chat_info_new (TLS, chat, &partner->user);
+      break;
+    }
+      
+    case TGL_PEER_USER:
+      return p2tgl_notify_user_info_new (&P->user);
+      break;
+      
+    default:
+      return purple_notify_user_info_new ();
+  }
+}
+
+int p2tgl_imgstore_add_with_id (const char* filename) {
+  gchar *data = NULL;
+  size_t len;
+  GError *err = NULL;
+  g_file_get_contents (filename, &data, &len, &err);
+  
+  int id = purple_imgstore_add_with_id (data, len, NULL);
+  return id;
+}
+
+void p2tgl_buddy_icons_set_for_user (PurpleAccount *pa, tgl_peer_id_t *id, const char* filename) {
+  char *who = g_strdup_printf("%d", tgl_get_peer_id(*id));
+
+  gchar *data = NULL;
+  size_t len;
+  GError *err = NULL;
+  g_file_get_contents (filename, &data, &len, &err);
+  
+  purple_buddy_icons_set_for_user (pa, who, data, len, NULL);
+  
+  g_free (who);
+}
+
